/** @license ISC License (c) copyright 2018 original and current authors */
/** @author Ian Hofmann-Hicks (evil) */

var curry = require('../core/curry')
var isArray = require('../core/isArray')
var isEmpty = require('../core/isEmpty')
var isInteger = require('../core/isInteger')
var isObject = require('../core/isObject')
var isString = require('../core/isString')
var object = require('../core/object')

var isValid = function (x) { return isObject(x) || isArray(x); }

var pathError =
  'setPath: Non-empty Array of non-empty Strings and/or Integers required for first argument'

// setPath :: [ String | Integer ] -> a -> (Object | Array) -> (Object | Array)
function setPath(path, val, obj) {
  var obj$1;

  if(!isArray(path) || isEmpty(path)) {
    throw new TypeError(pathError)
  }

  if(!isValid(obj)) {
    throw new TypeError('setPath: Object or Array required for third argument')
  }

  var key = path[0]
  var newVal = val

  if(path.length > 1) {
    var next = !isValid(obj[key])
      ? isInteger(path[1]) ? [] : {}
      : obj[key]

    newVal = setPath(path.slice(1), val, next)
  }

  if(isString(key) && !isEmpty(key)) {
    return object.assign(( obj$1 = {}, obj$1[key] = newVal, obj$1 ), obj)
  }

  if(isInteger(key)) {
    var arr =
      isArray(obj) ? obj.slice() : obj

    arr[key] = newVal

    return arr
  }

  throw new TypeError(pathError)
}

module.exports = curry(setPath)
