/** @license ISC License (c) copyright 2018 original and current authors */
/** @author Ian Hofmann-Hicks (evil) */

var curry = require('../core/curry')
var isArray = require('../core/isArray')
var isEmpty = require('../core/isEmpty')
var isInteger  = require('../core/isInteger')
var isObject  = require('../core/isObject')
var isString = require('../core/isString')
var object = require('../core/object')

var pathError =
  'unsetPath: Non-empty Array of non-empty Strings and/or Integers required for first argument'

function unset(key, obj) {
  return function(acc, k) {
    if(obj[k] !== undefined && k !== key) {
      acc[k] = obj[k]
    }

    return acc
  }
}

function unsetPath(path, obj) {
  var obj$1;

  if(!isArray(path) || isEmpty(path)) {
    throw new TypeError(pathError)
  }

  if(!(isObject(obj) || isArray(obj))) {
    throw new TypeError('unsetPath: Object or Array required for second argument')
  }

  var key = path[0]

  if(path.length === 1) {
    if(!(isString(key) || isInteger(key))) {
      throw new TypeError(pathError)
    }

    return isInteger(key) && isArray(obj)
      ? obj.slice(0, key).concat(obj.slice(key + 1))
      : Object.keys(obj).reduce(unset(key, obj), {})
  }

  var next =
    obj[key]

  if(!(isObject(next) || isArray(next))) {
    return obj
  }

  return isInteger(key) && isArray(obj)
    ? obj.slice(0, key)
      .concat([ unsetPath(path.slice(1), next) ])
      .concat(obj.slice(key + 1))
    : object.assign(( obj$1 = {}, obj$1[key] = unsetPath(path.slice(1), next), obj$1 ), obj)
}

module.exports = curry(unsetPath)
