/** @license ISC License (c) copyright 2018 original and current authors */
/** @author Ian Hofmann-Hicks (evilsoft) */

var curry = require('../core/curry')
var isArray = require('../core/isArray')
var isEmpty  = require('../core/isEmpty')
var isInteger = require('../core/isInteger')
var isNil = require('../core/isNil')
var isPredOrFunc = require('../core/isPredOrFunc')
var isString = require('../core/isString')
var predOrFunc = require('../core/predOrFunc')

// propPathSatisfies: [ (String | Integer) ] -> (a -> Boolean) -> b -> Boolean
// propPathSatisfies: [ (String | Integer) ] -> Pred a -> b -> Boolean
function propPathSatisfies(keys, pred, x) {
  if(!isArray(keys)) {
    throw new TypeError(
      'propPathSatisfies: Array of Non-empty Strings or Integers required for first argument'
    )
  }

  if(!isPredOrFunc(pred)) {
    throw new TypeError(
      'propPathSatisfies: Pred or predicate function required for second argument'
    )
  }

  if(isNil(x)) {
    return false
  }

  var target = x
  for(var i = 0; i < keys.length; i++) {
    var key = keys[i]

    if(!(isString(key) && !isEmpty(key) || isInteger(key))) {
      throw new TypeError(
        'propPathSatisfies: Array of Non-empty Strings or Integers required for first argument'
      )
    }

    if(isNil(target)) {
      return false
    }

    target = target[key]
  }

  return !!predOrFunc(pred, target)
}

module.exports = curry(propPathSatisfies)
